#' Get the top n genes from Seurat
#'
#' @param res
#' @param n The number of top genes to select
#'
#' @return
#'
#' @details ave_logFC
#'
get_top_genes_seurat <- function(output, n) {

  stopifnot(output$pars$method == "seurat")

  genes <- output$result
  top_genes <- dplyr::group_by(genes, cluster)

  # There are *no* p-values calculated for the ROC test.
  if (output$pars$test.use == "roc") {
    top_genes <- dplyr::arrange(top_genes, myAUC, avg_logFC)
  } else {
    top_genes <- dplyr::arrange(top_genes, p_val, avg_logFC)
  }

  top_genes <- dplyr::slice(top_genes, 1:n)
  top_genes <- dplyr::group_split(top_genes)

  out <- lapply(top_genes, function(x) x$gene)
  names(out) <- levels(genes$cluster)
  out
}

# TODO: Document
get_top_genes_scran <- function(output, n) {

  stopifnot(output$pars$method == "scran")

  genes <- output$result
  n_clusters <- length(genes)

  top_genes <- lapply(genes, function(x) {

    x <- as.data.frame(x)

    if (output$pars$pval.type == "any") {
      # TODO: Think more about this:
      # Empirically, this produces approximately the right number of genes.
      out <- filter(x, Top < (n / n_clusters) * 2)
    } else {
      out <- dplyr::arrange(x, p.value)
      out <- dplyr::slice(x, 1:n)
    }

    rownames(out)
  })

  top_genes
}

get_top_genes <- function(output, n) {

  out <- switch(
    output$pars$method,
    "scran" = get_top_genes_scran(output, n),
    "seurat" = get_top_genes_seurat(output, n),
    stop("Invalid method", call. = FALSE)
  )

  out
}
